PastureSystem = {
    defaultId = "Default"
}

local PastureSystem_mt = Class(PastureSystem)

function PastureSystem.new(customMt)
	local self = setmetatable({}, customMt or PastureSystem_mt)
    self.extendablePastures = {}
	self.pastureExtensions = {}
    self.mapLoadingFinished = false

	return self
end

function PastureSystem:addPastureExtension(pastureExtension, pastureSystemId, skipRemove)
    if pastureExtension == nil then
		Logging.error("No given pastureExtension")
		return
	end

    if not skipRemove then
        self:removePastureExtension(pastureExtension)
    end

    local id = pastureSystemId or PastureSystem.defaultId

    pastureExtension.parentPasture = id
    local currentPastureExtensions = self.pastureExtensions[id] or {}
	table.addElement(currentPastureExtensions, pastureExtension)
    self.pastureExtensions[id] = currentPastureExtensions

    local extendablePasture = self.extendablePastures[id]

    if extendablePasture ~= nil then
        extendablePasture:addExtension(pastureExtension, self.mapLoadingFinished)
    end
end

function PastureSystem:removePastureExtension(pastureExtension)
    local parentPastureId = pastureExtension.parentPasture or PastureSystem.defaultId
	table.removeElement(self.pastureExtensions[parentPastureId], pastureExtension)

    local parentPasture = self.extendablePastures[parentPastureId]

    if parentPasture ~= nil then
        parentPasture:removeExtension(pastureExtension, self.mapLoadingFinished)
    end
end

function PastureSystem:getPastureExtensionsForPasture(pastureSystemId)
    local pastureExtensions = self.pastureExtensions[pastureSystemId] or {}
    return pastureExtensions
end

function PastureSystem:addExtendablePasture(pasture)
	if pasture == nil then
		Logging.error("No given pasture")
		return
	end

    if pasture.pastureSystemId == nil then
        local freeId = 1
        while(self.extendablePastures[tostring(freeId)] ~= nil) do
            freeId = freeId + 1
        end

        pasture.pastureSystemId = tostring(freeId)
    end

    self.extendablePastures[pasture.pastureSystemId] = pasture

    if self.pastureExtensions[pasture.pastureSystemId] ~= nil then
        for k,v in ipairs(self.pastureExtensions[pasture.pastureSystemId]) do
            pasture:addExtension(v, self.mapLoadingFinished)
        end
    end
end

function PastureSystem:removeExtendablePasture(pasture)
    if self.extendablePastures[pasture.pastureSystemId] ~= nil then
        self.extendablePastures[pasture.pastureSystemId] = nil
    end
end

function PastureSystem:availablePasturesNameAndId(withoutPastureSystemId)
    local availablePastures = {}

    local playerFarm = g_farmManager:getFarmByUserId(g_currentMission.playerUserId)

    for k,v in pairs(self.extendablePastures) do
        if v:getOwnerFarmId() == playerFarm.farmId and v.pastureSystemId ~= withoutPastureSystemId then
            local value = {
                pastureName = v:getName(),
                pastureSystemId = v.pastureSystemId
            }
            table.addElement(availablePastures, value)
        end
    end

    return availablePastures
end

function PastureSystem:availablePastures(withoutPastureSystemId)
    local availablePastures = {}

    local playerFarm = g_farmManager:getFarmByUserId(g_currentMission.playerUserId)

    for k,v in pairs(self.extendablePastures) do
        if v:getOwnerFarmId() == playerFarm.farmId then
            if v.pastureSystemId ~= withoutPastureSystemId then
                table.addElement(availablePastures, v)
            end
        end
    end

    return availablePastures
end

function PastureSystem:allAllocatedPastureIds()
    local allocatedPastureSystemIds = {}

    for k,v in pairs(self.extendablePastures) do
        table.addElement(allocatedPastureSystemIds, v.pastureSystemId)
    end

    return allocatedPastureSystemIds
end

function PastureSystem:allAvailablePastureExtensions()
    local availablePastureExtensions = {}

    local playerFarm = g_farmManager:getFarmByUserId(g_currentMission.playerUserId)

    for k,v in pairs(self.pastureExtensions) do
        for i, pastureExtension in ipairs(v) do
            if pastureExtension:getOwnerFarmId() == playerFarm.farmId then
                table.addElement(availablePastureExtensions, pastureExtension)
            end
        end
    end

    return availablePastureExtensions
end

function PastureSystem:pastureWithId(id)
    return self.extendablePastures[id]
end

function PastureSystem:reloadAllPastures()
    local value = 0
    for k,v in pairs(self.extendablePastures) do
        if #self:getPastureExtensionsForPasture(v.pastureSystemId) > 0 then
            v:reloadNavMesh()
        end
        value = value + 1
    end
    Logging.info("Sync all "..value.." pasture navMeshes finished")
end